/*
 * Configuation settings for the EMAC SoM-5282 board.
 *
 * (C) Copyright 2005 EMAC.Inc www.emacinc.com
 *
 * See file CREDITS for list of people who contributed to this
 * project.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston,
 * MA 02111-1307 USA
 */

/*
 * board/config.h - configuration options, board specific
 */

#ifndef _CONFIG_SOM5282EM_H
#define _CONFIG_SOM5282EM_H

/**
 * High Level Configuration Options
 * (easy to change)
 */
#define	CONFIG_MCF52x2					/* define processor family */
#define    CONFIG_M5282						/* define processor type */
#define    CONFIG_SOM5282EM			/* define board type 
																		(currently used by cpu/serial.c)*/

/**
 * ethernet configuration
 */
//#define ET_DEBUG
//#define DEBUG
#define FEC_ENET
#define CFG_DISCOVER_PHY
#define  CFG_ENET_BD_BASE	0x380000
#define CONFIG_NET_RETRY_COUNT 5

/**
 * Board specific compile options
 * changing the these paramaters will also require manually 
 * changing the TEXT address in boards/SoM/5282EM/config.mk
 * This will require a make clean to obtain a correct image
 */

//#define INTERNAL_BOOT					//boot from internal flash(defaults to external)
#undef INTERNAL_BOOT						//boot from external flash
//#define FOURMEG								//Four megabyte flash (defaults to two)
#undef FOURMEG								//Two megabyte flash
 
 
#define CONFIG_WATCHDOG_TIMEOUT 3905 /* Should be ~500ms */
#define CONFIG_BAUDRATE 115200 
#define CFG_BAUDRATE_TABLE { 9600 , 19200 , 38400 , 57600, 115200 }
#define CONFIG_IPADDR 10.0.2.3

#define CONFIG_COMMANDS  (( CONFIG_CRC32_VERIFY|CONFIG_CMD_DFL | CFG_CMD_PING | CFG_CMD_MII | CFG_CMD_JFFS2 | CFG_AUTO_COMPLETE) & ~(CFG_CMD_LOADS | CFG_CMD_LOADB | CFG_FLASH_PROTECTION))

#define CFG_JFFS2_FIRST_BANK 0
#define CFG_JFFS2_NUM_BANKS 1

/* this must be included AFTER the definition of CONFIG_COMMANDS (if any) */
#include <cmd_confdefs.h>
#define CONFIG_BOOTDELAY	5

#ifdef INTERNAL_BOOT
#define CFG_PROMPT		"U-Boot-I> "
#else
#define CFG_PROMPT		"U-Boot-X> "
#endif

#define	CFG_LONGHELP				/* undef to save memory		*/

#if (CONFIG_COMMANDS & CFG_CMD_KGDB)
#define	CFG_CBSIZE		1024		/* Console I/O Buffer Size	*/
#else
#define	CFG_CBSIZE		256		/* Console I/O Buffer Size	*/
#endif
#define	CFG_PBSIZE (CFG_CBSIZE+sizeof(CFG_PROMPT)+16) /* Print Buffer Size */
#define	CFG_MAXARGS		16		/* max number of command args	*/
#define CFG_BARGSIZE		CFG_CBSIZE	/* Boot Argument Buffer Size	*/

#define CFG_LOAD_ADDR		0x10000

#define CFG_MEMTEST_START	0x400
#define CFG_MEMTEST_END		0x380000


#define CFG_HZ			1000000
#define CFG_CLK			64000000
#define MHZCLOCK		64		/* system bus frequency in MHz */


/*
 * Low Level Configuration Settings
 * (address mappings, register initial values, etc.)
 * You should know what you are doing if you make changes here.
 */
#define	CFG_MBAR		0x40000000

/*-----------------------------------------------------------------------
 * Definitions for initial stack pointer and data area (in DPRAM)
 */
#define CFG_INIT_RAM_ADDR       0x20000000
#define CFG_INIT_RAM_END	0x10000		/* End of used area in internal SRAM	*/
#define CFG_GBL_DATA_SIZE	64      	/* size in bytes reserved for initial data */
#define CFG_GBL_DATA_OFFSET	(CFG_INIT_RAM_END - CFG_GBL_DATA_SIZE)
#define CFG_INIT_SP_OFFSET	CFG_GBL_DATA_OFFSET

/*-----------------------------------------------------------------------
 * Start addresses for the final memory configuration
 * (Set up by the startup code)
 */
#define CFG_SDRAM_BASE			0x0000000
//#define CFG_SDRAM_BASE		0xff000000
#define	CFG_SDRAM_SIZE		16		/* SDRAM size in MB */


/*-----------------------------------------------------------------------
 * FLASH organization
 */
 
#ifdef FOURMEG
#define CFG_FLASH_BASE		0xffc00000
#define CFG_CSAR0 0xffc0
#define CFG_CSMR0 0x003f0001
#else //assumes 2M
#define CFG_FLASH_BASE		0xffe00000
#define CFG_CSAR0 0xffe0
#define CFG_CSMR0 0x001f0001
#endif

#define CFG_CSCR0 0x1980
 
#define CFG_INT_FLASH_BASE	0xf0000000
#define CFG_FLASH_CFI 1
#define CFG_FLASH_CFI_DRIVER 1
#define CFG_FLASH_CFM_DRIVER 1
#define CFG_FLASH_BANKS_LIST { CFG_FLASH_BASE, CFG_INT_FLASH_BASE }
#define CFG_MAX_FLASH_SECT      256
#define	CFG_MAX_FLASH_BANKS	2
#define	CFG_FLASH_ERASE_TOUT 	10000000


/* If M5282 port is fully implemented the monitor base will be behind
 * the vector table. */

#ifdef INTERNAL_BOOT
#define CFG_MONITOR_BASE	(CFG_INT_FLASH_BASE + 0x400) 
#else
#define CFG_MONITOR_BASE	(CFG_FLASH_BASE + 0x400) 
#endif

#define CFG_MONITOR_LEN		0x20000
#define CFG_MALLOC_LEN		(256 << 10)
#define CFG_BOOTPARAMS_LEN	64*1024


/*
 * For booting Linux, the board info and command line data
 * have to be in the first 8 MB of memory, since this is
 * the maximum mapped by the Linux kernel during initialization ??
 */
#define	CFG_BOOTMAPSZ		(8 << 20)	/* Initial Memory map for Linux	*/


/*-----------------------------------------------------------------------
 * Enviornmental Variables
 */
 #ifdef INTERNAL_BOOT
#define CFG_ENV_ADDR		0xf0020000 //internal flash
#define CFG_ENV_SIZE		(2*1024)
#define CFG_ENV_SECT_SIZE       (2*1024)
#else
//#define CFG_ENV_ADDR		0xffe20000
#define CFG_ENV_ADDR		(CFG_FLASH_BASE+0x20000)
#define CFG_ENV_SIZE		0x10000
#define CFG_ENV_SECT_SIZE   0x10000
#endif

#define CFG_ENV_IS_IN_FLASH	1
#define CFG_FLASH_CHECKSUM
//#define DEBUG_ENV

/*-----------------------------------------------------------------------
 * Cache Configuration
 */
#define CFG_CACHELINE_SIZE	16


/*-----------------------------------------------------------------------
 * Port configuration
 */
#define CFG_PEHLPAR	0xC0  /* Enable ethernet signals */
#define CFG_PUAPAR	0x0F  /* Enable UART */
#define CFG_DDRUA	0x05  /* Set DDR for UART */
#define CFG_PBCDPAR  0xC0  
#define CFG_PJPAR  0xFF  
#define CFG_PASPAR  0x0f0A  

#endif	/* _CONFIG_SOM5282EM_H */
